from setuptools import find_packages
from setuptools import setup

MAJOR_VERSION = '0'
MINOR_VERSION = '6'
MICRO_VERSION = '93'
VERSION = "{}.{}.{}".format(MAJOR_VERSION, MINOR_VERSION, MICRO_VERSION)

setup(
    name='just',
    version=VERSION,
    description="Automatically just read and write files based on extension.",
    author='Pascal van Kooten',
    url='https://github.com/kootenpv/just',
    author_email='kootenpv@gmail.com',
    install_requires=[
        'pyyaml',
        'dsv',
        'glob2',
        'dill',
        "jsonpath_rw",
        "preconvert",
        "preconvert_numpy",
    ],
    entry_points={'console_scripts': ['just = just.__main__:main']},
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Customer Service',
        'Intended Audience :: System Administrators',
        'Operating System :: Microsoft',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
    ],
    license='MIT',
    packages=find_packages(),
    zip_safe=False,
    platforms='any',
)



import time
import json

session = None


def _retry(request_fn, max_retries, delay_base, raw, kwargs):
    from requests import RequestException

    tries = 0
    # if 'headers' not in kwargs:
    #     kwargs["headers"] = {}
    # if 'User-Agent' not in kwargs:
    #     kwargs["headers"]['User-Agent'] = 'Just Agent 1.0'
    while tries < max_retries:
        try:
            r = request_fn(timeout=delay_base, **kwargs)
            if r.status_code > 399:
                return None
            break
        except RequestException as e:
            tries += 1
            print("just.requests_", kwargs["url"], "attempt", tries, str(e))
            if tries == max_retries:
                return ""
            time.sleep(delay_base ** tries)
    if raw:
        return r.content
    if "application/json" in r.headers['Content-Type']:
        return r.json()
    return r.text


def get(url, params=None, max_retries=1, delay_base=3, raw=False, **kwargs):
    import requests

    global session
    if session is None:
        session = requests.Session()

    kwargs['url'] = url
    if params is not None:
        kwargs['params'] = params
    result = _retry(session.get, max_retries, delay_base, raw, kwargs)
    return result


def post(url, params=None, data=None, max_retries=5, raw=False, json=None, delay_base=3, **kwargs):
    import requests

    global session
    if session is None:
        session = requests.Session()

    kwargs['url'] = url
    if params is not None:
        kwargs['params'] = params
    if data is not None:
        kwargs["data"] = data
    if json is not None:
        kwargs["json"] = json

    result = _retry(session.post, max_retries, delay_base, raw, kwargs)
    return result



from jsonpath_rw import parse


def json_extract(dc, expr):
    res = parse(expr).find(dc)
    if len(res) == 1:
        res = res[0].value
    else:
        res = [x.value for x in res]
    return res
